@@ -6,6 +6,8 @@ use warnings;
 
 use base qw/DBIx::Class/;
 
+__PACKAGE__->mk_classdata( 'strict_mode' => 1 );
+
 __PACKAGE__->mk_classdata( 'parent_virtual_column' => 'parent' );
 
 __PACKAGE__->mk_classdata( '_mobius_a_column' => 'mobius_a' );
@@ -45,18 +47,24 @@ sub add_mobius_tree_columns {
             "foreign.".$class->_mobius_c_column => "self.".$class->_mobius_d_column,
         });
 
-        $class->has_many( 'children' => $1 => {
+        $class->has_many( '_children' => $1 => {
             "foreign.".$class->_mobius_b_column => "self.".$class->_mobius_a_column,
             "foreign.".$class->_mobius_d_column => "self.".$class->_mobius_c_column,
         }, { cascade_delete => 0 });
       
     }
 
+
+}
+
+sub children {
+    my $self = shift;
+    return $self->is_leaf ? ( wantarray ? () : $self->result_source->resultset->search({ 0 => 1 }) ) : $self->_children(@_);
 }
 
 sub root_cond {
     my $self = shift;
-    return ( $self->_mobius_b_column => undef, $self->_mobius_d_column => undef );
+        return ( $self->_mobius_b_column => 1, $self->_mobius_d_column => undef );
 }
 
 sub inner_cond {
@@ -110,7 +118,13 @@ sub _left_right {
     my ($a, $b, $c, $d) = @_;
     my ($x, $y) = (($a+$b)/($c+$d), $a / $c);
     my ($left, $right) = $x > $y ? ($y, $x) : ($x, $y);
-    warn("DBIx::Class::Tree::Mobius max depth has been reached.") if ($left == $right);
+    if ($left == $right) {
+        if (__PACKAGE__->strict_mode) {
+            die("max depth has been reached.");
+        } else {
+            warn("max depth has been reached.");
+        }
+    }
     return wantarray ? ($left, $right) : sprintf("l=%.3f, r=%.3f", $left, $right);
 }
 
@@ -133,7 +147,7 @@ sub new {
 sub _available_mobius_index {
     my @children = @_;
 
-    my $count = scalar @children + 2;
+    my $count = scalar @children + 3;
     foreach my $child (@children) {
         my @mpath = $child->mobius_path();
         my $index = pop @mpath;
@@ -143,55 +157,107 @@ sub _available_mobius_index {
     return $count;
 }
 
+sub _mobius_parent {
+    my $self = shift;
+    return $self->parent || $self->result_source->resultset->new({});
+}
+
+sub _mobius_children {
+    my $self = shift;
+    return $self->in_storage ? $self->children : $self->result_source->resultset->search( { $self->root_cond } );
+}
+
 sub available_mobius_index {
     my $self = shift;
-    return _available_mobius_index( $self->children()->search({}, { order_by => $self->_mobius_a_column. ' DESC' } ) );
+        return _available_mobius_index( $self->_mobius_children->search({ $self->_is_inner_column => 1 }, { order_by => $self->_mobius_a_column. ' DESC' } ) );
+}
+
+sub _child_encoding {
+    my ($x, $pa, $pb, $pc, $pd) = @_;
+    
+    my ($a, $c) = ($pa * $x + $pb, $pc * $x + $pd);
+    my ($b, $d) = ($pa, $pc);
+
+    return wantarray ? ($a, $b, $c, $d, _left_right($a, $b, $c, $d)) : sprintf("(${a}x + $b) / (${c}x + $d)");
+}
+        
+sub child_encoding {
+    my $self = shift;
+    my $x = shift;
+
+    die ('cannot calculate encoding for non inner node') if ( $self->in_storage and not $self->get_column($self->_is_inner_column) );
+
+    my ($pa, $pc) = $self->in_storage ? ( $self->get_column($self->_mobius_a_column), $self->get_column($self->_mobius_c_column) ) : ( 1, 0 );
+    my ($pb, $pd) = $self->in_storage ? ( $self->get_column($self->_mobius_b_column), $self->get_column($self->_mobius_d_column) ) : ( 0, 1 );
+    return _child_encoding( $x, $pa, $pb, $pc, defined($pd) ? $pd : 0);
+}
+
+sub _abcd {
+    my $self = shift;
+
+    # matrix for mathematic root node 
+    my ($a, $b, $c, $d) = ( 1, 0, 0, 1 );
+
+    if ( $self->in_storage ) {
+
+        if ( $self->is_leaf ) {
+
+            ($a, $b, $c, $d) = $self->_mobius_parent->child_encoding( 2 );
+
+        } else {
+        
+            ($a, $b, $c, $d) = (
+                $self->get_column($self->_mobius_a_column),
+                $self->get_column($self->_mobius_b_column),
+                $self->get_column($self->_mobius_c_column),
+                $self->get_column($self->_mobius_d_column) // 0,
+                );
+
+        }
+
+    }
+
+    return wantarray ? ($a, $b, $c, $d) : sprintf("(${a}x + $b) / (${c}x + $d)");
+}
+
+sub mobius_path {
+    my $self = shift;
+    my @path = _mobius_path( $self->_abcd );
+    return wantarray ? @path : join('.', @path);
 }
 
 sub insert {
     my $self = shift;
 
-    if (exists $self->{_explicit_parent}
-        and my $parent = $self->result_source->resultset->find($self->{_explicit_parent}) ) {
+    my $parent = $self; # default parent to virtual mathematic super root node ( no SQL record associated )
 
-        my ($a, $b, $c, $d, $left, $right) = $parent->child_encoding( $parent->available_mobius_index );
+    if (exists $self->{_explicit_parent}) {
 
+        $parent = $self->result_source->resultset->find( $self->{_explicit_parent} );
+        $parent->make_inner_node();
+
+    } 
+
+    # mobius index 2 is the default encoding for all leaves
+    my ($a, $b, $c, $d, $left, $right) = $parent->child_encoding( $self->get_column($self->_is_inner_column) ? $parent->available_mobius_index : 2 );
+    
+    # a and c are not stored for leaves
+    if ($self->get_column($self->_is_inner_column)) {
         $self->store_column( $self->_mobius_a_column => $a );
-        $self->store_column( $self->_mobius_b_column => $b );
         $self->store_column( $self->_mobius_c_column => $c );
-        $self->store_column( $self->_mobius_d_column => $d );
-        $self->store_column( $self->_lft_column => $left );
-        $self->store_column( $self->_rgt_column => $right );
-
-        my $r = $self->next::method(@_);
-        $parent->update({ $self->_is_inner_column => 1 } );
-        return $r;
-
-    } else {  # attaching to root
-
-        my $x = _available_mobius_index( $self->result_source->resultset->search( { $self->root_cond } )->search({}, { order_by => $self->_mobius_a_column. ' DESC' } ) );
-
-        $self->store_column( $self->_mobius_a_column => $x );
-        $self->store_column( $self->_mobius_c_column => 1 );
-        # normal value are b => 1 and c => 0 but it cannot work for SQL join
-        $self->store_column( $self->_mobius_b_column => undef );
-        $self->store_column( $self->_mobius_d_column => undef );
-        $self->store_column( $self->_lft_column => $x );
-        $self->store_column( $self->_rgt_column => $x + 1 );
-        return $self->next::method(@_);
-
     }
 
-}
+    # d=0 (root nodes) is coded null to preserve unique contrainst
+    $self->store_column( $self->_mobius_d_column => ($d == 0) ? undef : $d );
+    $self->store_column( $self->_mobius_b_column => $b );
+    $self->store_column( $self->_lft_column => $left );
+    $self->store_column( $self->_rgt_column => $right );
+    
+    my $r = $self->next::method(@_);
 
-sub mobius_path {
-    my $self = shift;
-    my ($b, $d) =  ($self->get_column($self->_mobius_b_column), $self->get_column($self->_mobius_d_column));
-    my @path = _mobius_path(
-        $self->get_column($self->_mobius_a_column), defined $b ? $b : 1,
-        $self->get_column($self->_mobius_c_column), defined $d ? $d : 0,
-    );
-    return wantarray ? @path : join('.', @path);
+    $r->make_inner_node() unless ( $r->is_leaf );
+
+    return $r;
 }
 
 sub depth {
@@ -200,25 +266,12 @@ sub depth {
     return scalar @path;
 }
 
-sub child_encoding {
-    my $self = shift;
-    my $x = shift;
-    my ($pb, $pd) =  ($self->get_column($self->_mobius_b_column), $self->get_column($self->_mobius_d_column));
-    my ($a, $b, $c, $d) = (
-        $self->get_column($self->_mobius_a_column) * $x + ( defined $pb ? $pb : 1),
-        $self->get_column($self->_mobius_a_column),
-        $self->get_column($self->_mobius_c_column) * $x + ( defined $pd ? $pd : 0),
-        $self->get_column($self->_mobius_c_column)
-    );
-    return wantarray ? ($a, $b, $c, $d, _left_right($a, $b, $c, $d)) : sprintf("(${a}x + $b) / (${c}x + $d)");
-}
-
 sub root {
     my $self = shift;
-    return $self->result_source->resultset->search( { $self->root_cond } )->search({
+    return $self->parent ? $self->result_source->resultset->search( { $self->root_cond } )->search({
         $self->result_source->resultset->current_source_alias.'.'.$self->_lft_column => { '<' => $self->get_column($self->_rgt_column) },
         $self->result_source->resultset->current_source_alias.'.'.$self->_rgt_column => { '>' => $self->get_column($self->_lft_column) },
-    });
+                                                                                                   })->first : $self;
 }
 
 sub is_root {
@@ -241,25 +294,14 @@ sub is_leaf {
     return $self->get_column($self->_is_inner_column) ? 0 : 1;
 }
 
+# XXX can only work if primary key
 sub siblings {
     my $self = shift;
-    if (my $parent = $self->parent) {
-        return $parent->children->search({
-            -or => {
-                $self->result_source->resultset->current_source_alias.'.'.$self->_mobius_a_column => { '!=' => $self->get_column($self->_mobius_a_column) },
-                $self->result_source->resultset->current_source_alias.'.'.$self->_mobius_c_column => { '!=' => $self->get_column($self->_mobius_c_column) },
-            },
-        });
-    } else {
-        return $self->result_source->resultset->search({
-            -or => {
-                $self->result_source->resultset->current_source_alias.'.'.$self->_mobius_a_column => { '!=' => $self->get_column($self->_mobius_a_column) },
-                $self->result_source->resultset->current_source_alias.'.'.$self->_mobius_c_column => { '!=' => $self->get_column($self->_mobius_c_column) },
-            },
-            $self->result_source->resultset->current_source_alias.'.'.$self->_mobius_b_column => undef,
-            $self->result_source->resultset->current_source_alias.'.'.$self->_mobius_d_column => undef
-         });
-    }
+    return $self->_mobius_parent->_mobius_children();
+    # -or => {
+    #     $self->result_source->resultset->current_source_alias.'.'.$self->_mobius_a_column => { '!=' => $self->get_column($self->_mobius_a_column) },
+    #     $self->result_source->resultset->current_source_alias.'.'.$self->_mobius_c_column => { '!=' => $self->get_column($self->_mobius_c_column) },
+    # },
 }
 
 sub leaf_children {
@@ -306,35 +348,84 @@ sub ancestors {
         $self->result_source->resultset->current_source_alias.'.'.$self->_mobius_c_column => { '!=' => $self->get_column($self->_mobius_c_column) },
     },{ order_by => $self->_lft_column.' DESC' });
 }
-
 sub ascendants { return shift(@_)->ancestors(@_) }
 
-sub attach_child {
+sub make_inner_node {
+    my $self = shift;
+
+    if ( $self->in_storage and $self->is_leaf ) {
+
+        my $parent = $self->_mobius_parent;
+        my ($a, $b, $c, $d, $left, $right) = $parent->child_encoding( $parent->available_mobius_index );
+
+        $self->update({
+            $self->_is_inner_column => 1,
+            $self->_mobius_a_column => $a,
+            $self->_mobius_c_column => $c,
+            $self->_lft_column => $left,
+            $self->_rgt_column => $right,
+        });
+
+    }
+
+}
+
+sub _attach_child {
     my $self = shift;
     my $child = shift;
 
-    my ($a, $b, $c, $d, $left, $right) = $self->child_encoding( $self->available_mobius_index );
+    my @grandchildren = $child->children();
+
+    $self->make_inner_node();
+
+    my ($a, $b, $c, $d, $left, $right) = $self->child_encoding( $child->get_column($self->_is_inner_column) ? $self->available_mobius_index : 2 );
 
-    my @grandchildren = $child->children()->all();
     foreach my $grandchild (@grandchildren) {
         $grandchild->update( { $self->_mobius_b_column => undef, $self->_mobius_d_column => undef });
     }
 
+    # only store a/c for inner node
+    if ($child->get_column($self->_is_inner_column)) {
+        $child->set_column( $self->_mobius_a_column => $a );
+        $child->set_column( $self->_mobius_c_column => $c );
+    }
+
     $child->update({ 
-        $self->_mobius_a_column => $a,
         $self->_mobius_b_column => $b,
-        $self->_mobius_c_column => $c,
-        $self->_mobius_d_column => $d,
+        $self->_mobius_d_column => ($d == 0) ? undef : $d,
         $self->_lft_column => $left,
         $self->_rgt_column => $right,
     });
 
-    foreach my $grandchild (@grandchildren) {
-        $child->attach_child( $grandchild );
-    }
+    $child->attach_child( @grandchildren );
 
 }
 
+sub attach_child {
+    my $self = shift;
+
+    foreach my $child (@_) {
+
+        next if ( defined $child->get_column( $self->_mobius_b_column ) and defined  $child->get_column( $self->_mobius_d_column ) and
+                  $child->get_column( $self->_mobius_b_column ) == $self->get_column( $self->_mobius_a_column ) and
+                  $child->get_column( $self->_mobius_d_column ) == $self->get_column( $self->_mobius_c_column ));
+
+        $self->_attach_child( $child );
+
+    }
+}
+
+sub make_root {
+    my $self = shift;
+
+    next if ( $self->get_column( $self->_mobius_b_column ) == 1 and
+              not $self->get_column( $self->_mobius_d_column ) );
+    
+    $self->result_source->resultset->new({})->_attach_child( $self );
+
+}
+
+
 
 1;
 
@@ -413,7 +504,10 @@ Since the encoding is not volatile, the depth is constraint by the
 precision of FLOAT in the right and left column. The maximum depth
 reachable is 8 levels with a simple SQL FLOAT, and 21 with a SQL DOUBLE.
 
-This implementation allows you to have several trees in your database.
+This implementation allows you to have several root trees and corresponding
+trees in your database. To allow better performance, the Möbius encoding is
+the same for all non inner children of a given node. A unique Möbius encoding
+is calculated only if a node becomes 'inner'.
 
 =head1 METHODS
 
@@ -514,5 +608,14 @@ Given a mobius index, return the mobius a,b,c,d column values.
  	
 Return the depth of a node in a tree (depth of a root node is 1).
  	
-=for Pod::Coverage new mobius_path root_cond inner_cond leaf_cond
+=head2 make_root
+ 	
+Force a node to become a new tree root (if this node possess a subtree 
+of descendants, it becomes a new tree).
+ 	
+=head1 BUGS
+
+This version doesn't 
+
+=for Pod::Coverage new mobius_path root_cond inner_cond leaf_cond make_inner_node
 
